@@ -27,7 +27,7 @@ from .annotationparser.parser_utils import detect_type
 
 class Annotation:
 
-    def __init__(self, path, is_image=False):
+    def __init__(self, path, is_image=False, slidename=False):
         """Initialize the anntation object.
 
         Args:
@@ -37,6 +37,8 @@ class Annotation:
             is_image(bool): Whether the image is image.
         """
         self.path = path
+        self.slidename = slidename
+        self.dot_bbox_width = self.dot_bbox_height = 0
         self.is_image = is_image
         self.classes = []
         if not self.is_image:
@@ -50,9 +52,6 @@ class Annotation:
     def read_annotation(self, annotation_type=False):
         """Parse the annotation data.
 
-        WSIViewer is going to be added as a annotation type.
-        Currently, ASAP is available.
-
         Args:
             annotation_type (str): If provided, pass the auto type detection.
         """
@@ -60,8 +59,16 @@ class Annotation:
             annotation_type = detect_type(self.path)
         if annotation_type == "ASAP":
             from .annotationparser.ASAP_parser import AnnotationParser
+            parsed = AnnotationParser(self.path)
         elif annotation_type == "WSIDissector":
             from .annotationparser.wsidissector_parser import AnnotationParser
+            parsed = AnnotationParser(self.path)
+        elif annotation_type == "SlideRunner":
+            from .annotationparser.SlideRunner_parser import AnnotationParser
+            parsed = AnnotationParser(self.path, self.slidename)
+        elif annotation_type == "QuPath":
+            from .annotationparser.QuPath_parser import AnnotationParser
+            parsed = AnnotationParser(self.path)
         elif annotation_type == "Empty":
             class AnnotationParser:
                 classes = []
@@ -69,14 +76,40 @@ class Annotation:
 
                 def __init__(self, path):
                     print("Annotation File is Empty")
-        try:
             parsed = AnnotationParser(self.path)
-        except Exception as e:
-            raise NotImplementedError(
-                "[{e}] Could not parse {}".format(e, self.path))
         self.classes = parsed.classes
         self.mask_coords = parsed.mask_coords
 
+    def dot_to_bbox(self, width=30, height=False):
+        """Translate dot annotations to bounding boxes.
+
+        If the len(self.mask_coords[cls][idx]) is 1, the annotation is a dot.
+        And, the dot is the midpoint of the bounding box.
+
+        Args:
+            width (int): Width of the translated bounding box.
+            height (int): Height of the translated bounding box. If not set,
+                height is equal to width.
+        """
+        self.dot_bbox_width = width
+        self.dot_bbox_height = width if not height else height
+
+        for cls, coords in self.mask_coords.items():
+            for idx, coord in enumerate(coords):
+                if len(coord) == 1:
+                    center_x = coord[0][0]
+                    center_y = coord[0][1]
+                    left = int(center_x - self.dot_bbox_width / 2)
+                    top = int(center_y - self.dot_bbox_height / 2)
+                    right = int(center_x + self.dot_bbox_width / 2)
+                    bottom = int(center_y + self.dot_bbox_height / 2)
+                    lefttop = [left, top]
+                    righttop = [right, top]
+                    leftbottom = [left, bottom]
+                    rightbottom = [right, bottom]
+                    self.mask_coords[cls][idx] = [
+                        lefttop, righttop, rightbottom, leftbottom]
+
     def add_class(self, classes):
         for cls in classes:
             self.classes.append(cls)
@@ -118,15 +151,15 @@ class Annotation:
         """
         self.base_masks(slide.wsi_height, slide.wsi_width)
         self.main_masks()
+        if foreground:
+            self.make_foreground_mask(
+                slide, size, method=foreground, min_=min_, max_=max_)
         if rule:
             self.classes = list(set(self.classes) & set(rule.classes))
             self.include_masks(rule)
             self.merge_include_coords(rule)
             self.exclude_masks(rule)
-            self.exclude_coords(rule)
-        if foreground:
-            self.make_foreground_mask(
-                slide, size, method=foreground, min_=min_, max_=max_)
+            # self.exclude_coords(rule)
 
     def base_masks(self, wsi_height, wsi_width):
         """Make base masks.
@@ -159,7 +192,8 @@ class Annotation:
             contours = np.array(self.mask_coords[cls], dtype=object)
             for contour in contours:
                 self.masks[cls] = cv2.drawContours(
-                    self.masks[cls], [np.int32(contour)], 0, True, thickness=cv2.FILLED)
+                    self.masks[cls], [np.int32(contour)], 0, True,
+                    thickness=cv2.FILLED)
 
     def include_masks(self, rule):
         """Merge masks following the rule.
@@ -169,11 +203,13 @@ class Annotation:
         """
         self.masks_include = self.masks.copy()
         for cls in self.classes:
-            if hasattr(rule, cls):
-                for include in getattr(rule, cls)["includes"]:
-                    if include in self.masks:
-                        self.masks_include[cls] = cv2.bitwise_or(
-                            self.masks[cls], self.masks[include])
+            if not hasattr(rule, cls):
+                continue
+            for include in getattr(rule, cls)["includes"]:
+                if include not in self.masks:
+                    continue
+                self.masks_include[cls] = cv2.bitwise_or(
+                    self.masks[cls], self.masks[include])
         self.masks = self.masks_include
 
     def merge_include_coords(self, rule):
@@ -183,10 +219,12 @@ class Annotation:
             rule (wsiprocess.rule.Rule): Rule object.
         """
         for cls in self.classes:
-            if hasattr(rule, cls):
-                for include in getattr(rule, cls)["includes"]:
-                    if include in self.mask_coords.keys():
-                        self.mask_coords[cls].extend(self.mask_coords[include])
+            if not hasattr(rule, cls):
+                continue
+            for include in getattr(rule, cls)["includes"]:
+                if include not in self.mask_coords.keys():
+                    continue
+                self.mask_coords[cls].extend(self.mask_coords[include])
 
     def exclude_masks(self, rule):
         """Exclude area from base mask with following the rule.
@@ -196,13 +234,15 @@ class Annotation:
         """
         self.masks_exclude = self.masks.copy()
         for cls in self.classes:
-            if hasattr(rule, cls):
-                for exclude in getattr(rule, cls)["excludes"]:
-                    if exclude in self.masks:
-                        overlap_area = cv2.bitwise_and(
-                            self.masks[cls], self.masks[exclude])
-                        self.masks_exclude[cls] = cv2.bitwise_xor(
-                            self.masks[cls], overlap_area)
+            if not hasattr(rule, cls):
+                continue
+            for exclude in getattr(rule, cls)["excludes"]:
+                if exclude not in self.masks:
+                    continue
+                overlap_area = cv2.bitwise_and(
+                    self.masks[cls], self.masks[exclude])
+                self.masks_exclude[cls] = cv2.bitwise_xor(
+                    self.masks_exclude[cls], overlap_area)
         self.masks = self.masks_exclude
 
     def exclude_coords(self, rule):
@@ -212,12 +252,13 @@ class Annotation:
             rule (wsiprocess.rule.Rule): Rule object.
         """
         for cls in self.classes:
-            if hasattr(rule, cls):
-                base_set = [tuple(c) for c in self.mask_coords[cls]]
-                for exclude in getattr(rule, cls)["excludes"]:
-                    exclude_set = [tuple(c) for c in self.mask_coords[exclude]]
-                    base_set -= exclude_set
-                self.mask_coords[cls] = [list(c) for c in base_set]
+            if not hasattr(rule, cls):
+                continue
+            base_set = [tuple(c) for c in self.mask_coords[cls]]
+            for exclude in getattr(rule, cls)["excludes"]:
+                exclude_set = [tuple(c) for c in self.mask_coords[exclude]]
+                base_set -= exclude_set
+            self.mask_coords[cls] = [list(c) for c in base_set]
 
     def make_foreground_mask(
             self, slide, size=2000, method="otsu", min_=30, max_=190):
@@ -339,5 +380,6 @@ class Annotation:
             save_to (str): Parent directory to save the thumbnails.
             cls (str): Class name for each mask.
         """
-        cv2.imwrite(str(Path(save_to)/"{}.png".format(cls)),
-                    self.masks[cls], (cv2.IMWRITE_PXM_BINARY, 1))
+        cv2.imwrite(
+            str(Path(save_to)/"{}.png".format(cls)),
+            self.masks[cls], (cv2.IMWRITE_PXM_BINARY, 1))
