@@ -34,6 +34,14 @@ class Args:
         parser.add_argument(
             "-oy", "--offset_y", type=int, default=0,
             help="The offset pixel along the y-axis.")
+        parser.add_argument(
+            "-dw", "--dot_bbox_width", type=int, default=30,
+            help="Width of bbox translated from dot annotation."
+        )
+        parser.add_argument(
+            "-dh", "--dot_bbox_height", type=int,
+            help="Height of bbox translated from dot annotation."
+        )
         parser.add_argument(
             "-ss", "--start_sample", action="store_true",
             help="Generate samples at the start of the process.")
@@ -63,6 +71,9 @@ class Args:
         parser.add_argument(
             "-ru", "--rule", type=Path,
             help="File to define the inclusion / exclusion relationship.")
+        parser.add_argument(
+            "-ef", "--extract_foreground", action="store_true",
+            help="If set, wp extracts patches from foreground.")
         self.add_binarization_method(parser)
         self.add_on_foreground(parser, slide_is_sparse)
         self.add_on_annotation(parser, slide_is_sparse)
@@ -130,6 +141,10 @@ class Args:
         parser_det.add_argument(
             "-ra", "--ratio", default="8:1:1",
             help="Ratio of the dataset size of train/validation/test phase.")
+        parser_det.add_argument(
+            "-cb", "--crop_bbox", default=False, action="store_true",
+            help="Crop bounding boxes after patch extraction."
+        )
         self.add_annotation_args(parser_det, slide_is_sparse=True)
         self.set_common_args(parser_det)
 
@@ -159,6 +174,7 @@ class Args:
 def process_annotation(args, slide, rule):
     if args.method == "none":
         annotation = wp.annotation("")
+        annotation.dot_to_bbox(args.dot_bbox_width, args.dot_bbox_height)
         if args.minmax:
             min_, max_ = map(int, args.minmax.split("-"))
             annotation.make_masks(
@@ -168,14 +184,18 @@ def process_annotation(args, slide, rule):
                 slide, foreground="otsu")
 
     else:
-        annotation = wp.annotation(args.annotation)
+        annotation = wp.annotation(args.annotation, slidename=slide.filename)
+        annotation.dot_to_bbox(args.dot_bbox_width, args.dot_bbox_height)
         if hasattr(args, "minmax") and args.minmax:
             min_, max_ = map(int, args.minmax.split("-"))
             annotation.make_masks(
                 slide, rule, foreground="minmax", min_=min_, max_=max_)
         else:
             annotation.make_masks(slide, rule, foreground=True)
-        annotation.classes.remove("foreground")
+
+    if hasattr(args, "extract_foreground"):
+        if not (args.extract_foreground and "foreground" in annotation.classes):
+            annotation.classes.remove("foreground")
 
     return annotation
 
@@ -183,7 +203,7 @@ def process_annotation(args, slide, rule):
 def main(command=None):
     args = Args(command)
     slide = wp.slide(args.wsi)
-    rule = wp.rule(args.rule) if hasattr(args, "rule") else False
+    rule = wp.rule(args.rule) if hasattr(args, "rule") and args.rule else False
     annotation = process_annotation(args, slide, rule)
 
     if hasattr(args, "export_thumbs") and args.export_thumbs:
@@ -193,6 +213,7 @@ def main(command=None):
         annotation.export_thumb_masks(thumbs_dir)
 
     on_annotation = False if args.method == "none" else args.on_annotation
+    crop_bbox = args.crop_bbox if hasattr(args, "crop_bbox") else False
 
     patcher = wp.patcher(
         slide, args.method,
@@ -208,7 +229,8 @@ def main(command=None):
         offset_y=args.offset_y,
         start_sample=args.start_sample,
         finished_sample=args.finished_sample,
-        no_patches=args.no_patches)
+        no_patches=args.no_patches,
+        crop_bbox=crop_bbox)
     patcher.get_patch_parallel(annotation.classes)
 
     if args.method == "detection":
@@ -222,3 +244,6 @@ def main(command=None):
             converter.to_coco()
         if args.yolo_style:
             converter.to_yolo()
+
+        if args.crop_bbox:
+            patcher.get_mini_patch_parallel(annotation.classes)
